// BPALS.CPP


#include "src/bsuite.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


// Format of PALETTE.DAT files
//
//   256 bytes     - palette
//   2 bytes       - short: number of palette lookups
//   n*256 bytes   - shading lookup tables
//   256*256 bytes - translucency lookup array



int bsExtractPalette(void)
{
	FILE *palette, *out;
	char outfn[62];
	char cb[3];
	int lp;

	strcpy(outfn, "GAME.PAL");
	if (uiInput("Enter the filename to write to:", outfn, 60))
		return 1;

	palette = fopen("PALETTE.DAT", "rb");
	if (palette == NULL)
	{
		// could not open PALETTE.DAT
		uiMessage("~!C3Error opening PALETTE.DAT!~@\n"
							"Make sure that the file is in the current\n"
							"directory, then try again.\n\n"
							"Press any key to continue.");

		return 1;
	}

	// create output file
	out = fopen(outfn, "w+t");
	if (out == NULL)
	{
		fclose(palette);
		uiMessage("~!C3Error creating output file!~@\n"
							"The file may be open by another program\n"
							"or is read-only, or the disk may be read-only.\n\n"
							"Press any key to continue.");
		return 1;
	}

	// write out the palette data in PSP format

	// write out a Paint Shop Pro palette file
	fprintf(out, "JASC-PAL\n0100\n256\n");
	for (lp=0; lp < 256; lp++)
	{
		cb[0] = fgetc(palette) * 4;
		cb[1] = fgetc(palette) * 4;
		cb[2] = fgetc(palette) * 4;

		fprintf(out, "%d %d %d\n", cb[0], cb[1], cb[2]);
	}

	// close files
	fclose(out);
	fclose(palette);

	uiMessage("~!E3Palette dumped successfully!~@\n\n"
						"Press any key to continue.");

	return 0;
}


int bsUpdatePalette(void)
{
	FILE *palette, *pal;
	char palfn[62];
	char cb[3], work[64];
	int lp;

	strcpy(palfn, "GAME.PAL");
	if (uiInput("Enter the filename to update from:", palfn, 60))
		return 1;

	palette = fopen("PALETTE.DAT", "w+b");
	if (palette == NULL)
	{
		// could not open LOOKUP.DAT
		uiMessage("~!C3Error opening PALETTE.DAT!~@\n"
							"The file may be open by another program\n"
							"or is read-only, or the disk may be read-only.\n\n"
							"Press any key to continue.");

		return 1;
	}

	// open source file
	pal = fopen(palfn, "rt");
	if (pal == NULL)
	{
		fclose(palette);
		uiMessage("~!C3Error opening palette file!~@\n"
							"Make sure that the file exists.\n\n"
							"Press any key to continue.");
		return 1;
	}

	// read the palette data and write it to PALETTE.DAT

	fgets(work, 64, pal);
	if (strncmp(work, "JASC-PAL", 8))
	{
		fclose(pal); fclose(palette);
		uiMessage("~!C3Error validating palette file!~@\n"
							"This palette file appears to be in a format\n"
							"other than Paint Shop Pro format. This program\n"
							"works only with Paint Shop Pro palette files.\n\n"
							"Press any key to continue.");
		return 1;
	}

	fgets(work, 64, pal);
	if (strncmp(work, "0100", 4))
	{
		fclose(pal); fclose(palette);
		uiMessage("~!C3Error validating palette file!~@\n"
							"This palette file appears to be in a version\n"
							"other than 0100. This program works only with\n"
							"Paint Shop Pro palettes of version 0100.\n\n"
							"Press any key to continue.");
		return 1;
	}

	fgets(work, 64, pal);
	if (strncmp(work, "256", 3))
	{
		fclose(pal); fclose(palette);
		uiMessage("~!C3Error validating palette file!~@\n"
							"This palette file appears to be for a palette\n"
							"size other than 256 colours. This program works\n"
							"only with Paint Shop Pro palettes of 256 colours.\n\n"
							"Press any key to continue.");
		return 1;
	}

	// write out new palette info
	for (lp=0; lp < 256; lp++)
	{
		fscanf(pal, "%d %d %d\n", &cb[0], &cb[1], &cb[2]);

		// put the bytes into the basergb array as well
		fputc(cb[0] / 4, palette);
		fputc(cb[1] / 4, palette);
		fputc(cb[2] / 4, palette);
	}

	// close files
	fclose(pal);
	fclose(palette);

	uiMessage("~!E3Palette updated successfully!~@\n"
						"Now run TRANSPAL.EXE to create the shading\n"
						"and translucency tables for the palette.\n\n"
						"Press any key to continue.");

	return 0;
}