// D3DPAL.CPP


#include "src/bsuite.h"
#include <conio.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>


const int pal_offsets[5] = {6426, 7194, 7962, 8730, 9498};
const char *pal_deffn[5] = {
		"D3DWATER.PAL",
		"D3DNVIS.PAL",
		"D3DTITLE.PAL",
		"D3D3DR.PAL",
		"D3DEP1.PAL"
	};


#define PAL_LEN			768

/////////////////////////////////////////////////
//
// Menu Control

const char *D3DMenuStrings[] = {
		"~!E1W~@ater Palette",
		"~!E1N~@ight-Vision Palette",
		"~!E1T~@itle Screen Palette",
		"~!E13~@D Realms Logo Palette",
		"~!E1E~@pisode 1 Ending Animation Palette",
		"~!C1B~@ack to Main Menu"
	};
#define D3D_MENU_COUNT	6
#define D3D_MENU_POSX		18
#define D3D_MENU_POSY		7
#define D3D_MENU_WIDE		44
#define D3D_MENU_HIGH		11

char D3DMenuOptions[] = "BWNT3E";

void bsD3DExtractMenu_Paint(void)
{
	int lp;

	// draw the main menu window
	uiDrawDoubleBox(D3D_MENU_POSX, D3D_MENU_POSY,
		D3D_MENU_WIDE,D3D_MENU_HIGH,15,1);
	uiPrintString(D3D_MENU_POSX+2, D3D_MENU_POSY, " Extract Duke3D Palette ",14,1);

	// place the menu item strings
	for (lp=0; lp<D3D_MENU_COUNT; lp++)
	{
		if (lp == (D3D_MENU_COUNT-1))
			uiPrintString(D3D_MENU_POSX+4,D3D_MENU_POSY+2+lp+1,
				D3DMenuStrings[lp], 15,1);
		else
			uiPrintString(D3D_MENU_POSX+4,D3D_MENU_POSY+2+lp,
				D3DMenuStrings[lp], 15,1);
	}
}

int bsD3DExtractMenu(void)
{
	int option=0, key;
	char *op;

	bsD3DExtractMenu_Paint();

	do
	{
		key=toupper(getch());

		if ((op=strchr(D3DMenuOptions, key)) != NULL)
			option = op-D3DMenuOptions + 1;
		else
			putchar('\a');
	} while (!option);	// while an option is not selected

	// erase the menu
	uiClearRegion(D3D_MENU_POSX, D3D_MENU_POSY,
		D3D_MENU_WIDE,D3D_MENU_HIGH,0);

	return (option);
}

/////////

void bsD3DUpdateMenu_Paint(void)
{
	int lp;

	// draw the main menu window
	uiDrawDoubleBox(D3D_MENU_POSX, D3D_MENU_POSY,
		D3D_MENU_WIDE,D3D_MENU_HIGH,15,1);
	uiPrintString(D3D_MENU_POSX+2, D3D_MENU_POSY, " Update Duke3D Palette ",14,1);

	// place the menu item strings
	for (lp=0; lp<D3D_MENU_COUNT; lp++)
	{
		if (lp == (D3D_MENU_COUNT-1))
			uiPrintString(D3D_MENU_POSX+4,D3D_MENU_POSY+2+lp+1,
				D3DMenuStrings[lp], 15,1);
		else
			uiPrintString(D3D_MENU_POSX+4,D3D_MENU_POSY+2+lp,
				D3DMenuStrings[lp], 15,1);
	}
}

int bsD3DUpdateMenu(void)
{
	int option=0, key;
	char *op;

	bsD3DUpdateMenu_Paint();

	do
	{
		key=toupper(getch());

		if ((op=strchr(D3DMenuOptions, key)) != NULL)
			option = op-D3DMenuOptions + 1;
		else
			putchar('\a');
	} while (!option);	// while an option is not selected

	// erase the menu
	uiClearRegion(D3D_MENU_POSX, D3D_MENU_POSY,
		D3D_MENU_WIDE,D3D_MENU_HIGH,0);

	return (option);
}

/////////////////////////////////////////////////
//
// Extract and update functions

int bsExtractD3DPalette(int palnum)
{
	FILE *lookup, *out;
	char outfn[62];
	char cb[3];
	int lp;

	strcpy(outfn, pal_deffn[palnum]);
	if (uiInput("Enter the filename to write to:", outfn, 60))
		return 1;

	lookup = fopen("LOOKUP.DAT", "rb");
	if (lookup == NULL)
	{
		// could not open LOOKUP.DAT
		uiMessage("~!C3Error opening LOOKUP.DAT!~@\n"
							"Make sure that the file is in the current\n"
							"directory, then try again.\n\n"
							"Press any key to continue.");

		return 1;
	}

	// create output file
	out = fopen(outfn, "w+t");
	if (out == NULL)
	{
		fclose(lookup);
		uiMessage("~!C3Error creating output file!~@\n"
							"The file may be open by another program\n"
							"or is read-only, or the disk may be read-only.\n\n"
							"Press any key to continue.");
		return 1;
	}

	// write out the palette data in PSP format

	// find palette data
	fseek(lookup, pal_offsets[palnum], SEEK_SET);

	// write out a Paint Shop Pro palette file
	fprintf(out, "JASC-PAL\n0100\n256\n");
	for (lp=0; lp < 256; lp++)
	{
		cb[0] = fgetc(lookup) * 4;
		cb[1] = fgetc(lookup) * 4;
		cb[2] = fgetc(lookup) * 4;

		fprintf(out, "%d %d %d\n", cb[0], cb[1], cb[2]);
	}

	// close files
	fclose(out);
	fclose(lookup);

	uiMessage("~!E3Palette dumped successfully!~@\n\n"
						"Press any key to continue.");

	return 0;
}


int bsUpdateD3DPalette(int palnum)
{
	FILE *lookup, *pal;
	char palfn[62];
	char cb[3], work[64];
	int lp;

	strcpy(palfn, pal_deffn[palnum]);
	if (uiInput("Enter the filename to update from:", palfn, 60))
		return 1;

	lookup = fopen("LOOKUP.DAT", "r+b");
	if (lookup == NULL)
	{
		// could not open LOOKUP.DAT
		uiMessage("~!C3Error opening LOOKUP.DAT!~@\n"
							"The file may be open by another program\n"
							"or is read-only, or the disk may be read-only.\n\n"
							"Press any key to continue.");

		return 1;
	}

	// open source file
	pal = fopen(palfn, "rt");
	if (pal == NULL)
	{
		fclose(lookup);
		uiMessage("~!C3Error opening palette file!~@\n"
							"Make sure that the file exists.\n\n"
							"Press any key to continue.");
		return 1;
	}

	// read the palette data and write it to LOOKUP.DAT

	fgets(work, 64, pal);
	if (strncmp(work, "JASC-PAL", 8))
	{
		fclose(pal); fclose(lookup);
		uiMessage("~!C3Error validating palette file!~@\n"
							"This palette file appears to be in a format\n"
							"other than Paint Shop Pro format. This program\n"
							"works only with Paint Shop Pro palette files.\n\n"
							"Press any key to continue.");
		return 1;
	}

	fgets(work, 64, pal);
	if (strncmp(work, "0100", 4))
	{
		fclose(pal); fclose(lookup);
		uiMessage("~!C3Error validating palette file!~@\n"
							"This palette file appears to be in a version\n"
							"other than 0100. This program works only with\n"
							"Paint Shop Pro palettes of version 0100.\n\n"
							"Press any key to continue.");
		return 1;
	}

	fgets(work, 64, pal);
	if (strncmp(work, "256", 3))
	{
		fclose(pal); fclose(lookup);
		uiMessage("~!C3Error validating palette file!~@\n"
							"This palette file appears to be for a palette\n"
							"size other than 256 colours. This program works\n"
							"only with Paint Shop Pro palettes of 256 colours.\n\n"
							"Press any key to continue.");
		return 1;
	}

	// find palette data
	fseek(lookup, pal_offsets[palnum], SEEK_SET);

	// write out new palette info
	for (lp=0; lp < 256; lp++)
	{
		fscanf(pal, "%d %d %d\n", &cb[0], &cb[1], &cb[2]);

		fputc(cb[0] / 4, lookup);
		fputc(cb[1] / 4, lookup);
		fputc(cb[2] / 4, lookup);
	}

	// close files
	fclose(pal);
	fclose(lookup);

	uiMessage("~!E3Palette updated successfully!~@\n\n"
						"Press any key to continue.");

	return 0;
}